@@ -7,18 +7,14 @@ from sqlalchemy.orm import DeclarativeMeta
 from sqlalchemy.sql.schema import MetaData
 
 from sqlalchemy_declarative_extensions.grant.base import Grants
-from sqlalchemy_declarative_extensions.grant.ddl import grant_ddl
 from sqlalchemy_declarative_extensions.role.base import Roles
-from sqlalchemy_declarative_extensions.role.ddl import role_ddl
-from sqlalchemy_declarative_extensions.role.topological_sort import topological_sort
 from sqlalchemy_declarative_extensions.row.base import Row, Rows
-from sqlalchemy_declarative_extensions.row.query import rows_query
 from sqlalchemy_declarative_extensions.schema.base import Schemas
-from sqlalchemy_declarative_extensions.schema.ddl import schema_ddl
 
 if TYPE_CHECKING:
+    from sqlalchemy_declarative_extensions.dialects import postgresql
     from sqlalchemy_declarative_extensions.grant.base import G
-    from sqlalchemy_declarative_extensions.role.base import R_unknown
+    from sqlalchemy_declarative_extensions.role import generic
     from sqlalchemy_declarative_extensions.schema.base import Schema
 
 
@@ -40,7 +36,8 @@ def declarative_database(
         >>> from sqlalchemy.orm import declarative_base
         >>> Base_ = declarative_base()
         >>>
-        >>> from sqlalchemy_declarative_extensions import declarative_database, PGRole, Roles
+        >>> from sqlalchemy_declarative_extensions import declarative_database, Roles
+        >>> from sqlalchemy_declarative_extensions.dialects.postgresql import Role
         >>> @declarative_database
         ... class Base(Base_):
         ...     __abstract__ = True
@@ -50,7 +47,7 @@ def declarative_database(
         ...         Roles(ignore_unspecified=True)
         ...         .are(
         ...             'read',
-        ...             PGRole('fancy', createrole=True, in_roles=['read']),
+        ...             Role('fancy', createrole=True, in_roles=['read']),
         ...         )
         ...     )
 
@@ -80,8 +77,10 @@ def declare_database(
     metadata: MetaData,
     *,
     schemas: Union[None, Iterable[Union[Schema, str]], Schemas] = None,
-    roles: Union[None, Iterable[R_unknown], Roles] = None,
-    grants: Union[None, Iterable[G], Grants[G]] = None,
+    roles: Union[
+        None, Iterable[Union[generic.Role, postgresql.Role, str]], Roles
+    ] = None,
+    grants: Union[None, Iterable[G], Grants] = None,
     rows: Union[None, Iterable[Row], Rows] = None,
 ):
     """Register declaratively specified database extension handlers.
@@ -110,12 +109,45 @@ def declare_database(
         grants: The set of grants to ensure are applied to the roles/schemas/tables.
         rows: The set of rows to ensure are applied to the roles/schemas/tables.
     """
-    concrete_schemas = metadata.info["schemas"] = Schemas.coerce_from_unknown(schemas)
-    concrete_roles = metadata.info["roles"] = Roles.coerce_from_unknown(roles)
-    concrete_grants = metadata.info["grants"] = Grants.coerce_from_unknown(grants)
-    concrete_rows = metadata.info["rows"] = Rows.coerce_from_unknown(rows)
+    metadata.info["schemas"] = Schemas.coerce_from_unknown(schemas)
+    metadata.info["roles"] = Roles.coerce_from_unknown(roles)
+    metadata.info["grants"] = Grants.coerce_from_unknown(grants)
+    metadata.info["rows"] = Rows.coerce_from_unknown(rows)
 
-    if concrete_schemas:
+
+def register_sqlalchemy_events(
+    maybe_metadata: MetaData | DeclarativeMeta,
+    schemas=False,
+    roles=False,
+    grants=False,
+    rows=False,
+):
+    """Register handlers for supported object types into SQLAlchemy's event system.
+
+    By default all object types are disabled, but each can be individually enabled.
+    We assume most execution environments where one is using `MetaData.create_all`
+    will be in tests; where roles and grants, in particular, are database-wide
+    objects which can cause issues.
+
+    Note this is the opposite of the defaults when registering against SQLAlchemy's
+    event system.
+    """
+    from sqlalchemy_declarative_extensions.grant.ddl import grant_ddl
+    from sqlalchemy_declarative_extensions.role.ddl import role_ddl
+    from sqlalchemy_declarative_extensions.row.query import rows_query
+    from sqlalchemy_declarative_extensions.schema.ddl import schema_ddl
+
+    if isinstance(maybe_metadata, MetaData):
+        metadata = maybe_metadata
+    else:
+        metadata = maybe_metadata.metadata
+
+    concrete_schemas = metadata.info["schemas"]
+    concrete_roles = metadata.info["roles"]
+    concrete_grants = metadata.info["grants"]
+    concrete_rows = metadata.info["rows"]
+
+    if concrete_schemas and schemas:
         for schema in concrete_schemas:
             event.listen(
                 metadata,
@@ -123,28 +155,27 @@ def declare_database(
                 schema_ddl(schema),
             )
 
-    if concrete_roles:
-        for role in topological_sort(concrete_roles):
-            event.listen(
-                metadata,
-                "before_create",
-                role_ddl(role),
-            )
-
-    if concrete_grants:
+    if concrete_roles and roles:
         event.listen(
             metadata,
             "before_create",
-            grant_ddl(concrete_grants, default=True),
+            role_ddl,
+        )
+
+    if concrete_grants and grants:
+        event.listen(
+            metadata,
+            "before_create",
+            grant_ddl(concrete_grants, after=False),
         )
 
         event.listen(
             metadata,
             "after_create",
-            grant_ddl(concrete_grants, default=False),
+            grant_ddl(concrete_grants, after=True),
         )
 
-    if concrete_rows:
+    if concrete_rows and rows:
         event.listen(
             metadata,
             "after_create",
