@@ -1,12 +1,34 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, replace
-from typing import Iterable, Optional, Tuple, Union
+from typing import Iterable, Optional, Sequence, Union
 
 
 @dataclass(frozen=True)
 class Schemas:
-    schemas: Tuple[Schema, ...] = ()
+    """A collection of schemas and the settings for diff/collection.
+
+    Arguments:
+        schemas: The list of grants
+        ignore_unspecified: Optionally ignore detected grants which do not match
+            the set of defined grants.
+
+    Examples:
+        - No schemas
+
+        >>> schemas = Schemas()
+
+        - Some options set
+
+        >>> schemas = Schemas(ignore_unspecified=True)
+
+        - With some actual schemas
+
+        >>> from sqlalchemy_declarative_extensions import Schema, Schemas
+        >>> schema = Schemas().are("foo", Schema("bar"), ...)
+    """
+
+    schemas: Sequence[Schema] = ()
     ignore_unspecified: bool = False
 
     @classmethod
@@ -21,15 +43,12 @@ class Schemas:
 
         return None
 
-    @classmethod
-    def options(cls, ignore_unspecified=False):
-        return cls(ignore_unspecified=ignore_unspecified)
-
     def __iter__(self):
         for schema in self.schemas:
             yield schema
 
     def are(self, *schemas: Union[Schema, str]):
+        """Declare the set of schemas which should exist."""
         result = replace(
             self,
             schemas=tuple([Schema.coerce_from_unknown(schema) for schema in schemas]),
@@ -39,6 +58,8 @@ class Schemas:
 
 @dataclass(frozen=True, order=True)
 class Schema:
+    """Represents a schema."""
+
     name: str
 
     @classmethod
